from jennifer.agent import jennifer_agent
import json
__hooking_module__ = 'redis'


def format_command(*args):
    cmd = args[0]
    parameters = [cmd]
    for arg in args[1:]:
        p = json.dumps(arg)
        parameters.append(p)

    return ' [REDIS] ' + ' '.join(parameters)

def wrap_execute_command(origin):
    agent = jennifer_agent()
    def handler(self, *args, **kwargs):
        transaction = agent.current_transaction()
        if transaction is not None:
            try:
                message = format_command(*args)
                transaction.profiler.message(message)
            except:
                pass
        ret = origin(self, *args, **kwargs)
        return ret
    return handler


def hook(redis):
    redis.Redis.execute_command = wrap_execute_command(
        redis.Redis.execute_command)



from .profile_data import ProfileDataType


class Error(ProfileDataType):
    UNDEFINED = 0
    SERVICE_EXCEPTION = 1001
    OUT_OF_MEMORY = 1003
    DB_CONNECTION_UNCLOSED = 1004
    DB_STATEMENT_UNCLOSED = 1005
    DB_RESULTSET_UNCLOSED = 1006
    HTTP_IO_EXCEPTION = 1007
    SERVICE_ERROR = 1008
    PLC_REJECTED = 1009
    DEADLOCK = 1011
    METHOD_EXCEPTION = 1012
    SQL_TOOMANY_FETCH = 1013
    EXTERNALCALL_EXCEPTION = 1014
    DB_CONNECTION_FAIL = 1015
    DB_UNCOMMIT_ROLLBACK = 1016
    DB_CONNECTION_ILLEGAL_ACCESS = 1017
    RECURSIVE_CALL = 1018
    SQL_EXCEPTION = 1020
    HTTP_404_ERROR = 1022
    CORE_ERROR = 3000
    COMPILE_ERROR = 3001
    PARSE_ERROR = 3002
    NATIVE_CRITICAL_ERROR = 3003
    PHP_WARNING = 3004
    BAD_RESPONSE_TIME = 1002
    AGENT_STOP = 1019
    AGENT_START = 1021

    def __init__(
            self,
            error_type=0,
            error_hash=0,
    ):
        ProfileDataType.__init__(self)
        self.error_hash = error_hash
        self.error_type = error_type

    def get_type(self):
        return ProfileDataType.TYPE_ERROR



import os
import logging
try:
    from configparser import ConfigParser
except ImportError:
    import ConfigParser


class ConfigNotExistsError(Exception):
    pass


class Config(object):
    def __init__(self):
        self.config_file = os.environ.get('JENNIFER_CONFIG_FILE')
        if self.config_file is None:
            raise EnvironmentError("JENNIFER_CONFIG_FILE is not set")
        self.config_file = os.path.join(os.getcwd(), self.config_file)
        if not os.path.exists(self.config_file):
            raise FileNotFoundError("JENNIFER_CONFIG_FILE not exist")

        self.config = ConfigParser()
        self.config.read(self.config_file)  # Read config file
        self.logger = logging.getLogger('jennifer')

    def validate_config(self):
        # if self.server_address is None:
        #     raise ConfigNotExistsError('server_address is not found')
        # elif self.server_port is None:
        #     raise ConfigNotExistsError('server_port is not found')
        # elif self.domain_id is None:
        #     raise ConfigNotExistsError('domain_id is not found')
        # elif self.instance_id is None:
        #     raise ConfigNotExistsError('instance_id is not found')
        return

    def _get_config(self, item):
        try:
            return self.config.get('JENNIFER', item)
        except configparser.NoSectionError:
            return None
        except configparser.NoOptionError:
            return None

    @property
    def server_address(self):
        v = self._get_config('server_address')
        if v is None:
            return '127.0.0.1'
        return str(v)

    @property
    def server_port(self):
        v = self._get_config('server_port')
        if v is None:
            return 5000
        return int(v)

    @property
    def domain_id(self):
        v = self._get_config('domain_id')
        if v is None:
            return 1000
        return int(v)

    @property
    def instance_id(self):
        v = self._get_config('instance_id')
        if v is None:
            return -1
        return int(v)

    @property
    def log_path(self):
        v = self._get_config('log_path')
        if v is None:
            return '/tmp/jennifer-python-agent.log'
        return v

    def write_instance_id(self, inst_id):
        if inst_id is None or not isinstance(inst_id, int):
            logging.getLogger('jennifer').error('Can not write non-int inst_id')
        self.config.set('JENNIFER', 'instance_id', str(inst_id))
        self.save()

    def save(self):
        f = open(self.config_file, 'w')
        self.config.write(f)



from os import replace
from typing import Collection
from jennifer.api.proxy import Proxy
from jennifer.agent import jennifer_agent
import json

__hooking_module__ = 'pymongo'


def safe(l, idx, default=None):
    try:
        return l[idx]
    except IndexError:
        return default


class CollectionProxy(Proxy):
    def __init__(self, obj, host, port, name):
        Proxy.__init__(self, obj)
        self.set('collection', name)
        self.set('host', host)
        self.set('port', port)

    def with_options(self, *args, **kwargs):
        obj = self._origin.with_options(*args, **kwargs)
        return CollectionProxy(
            obj, self.host, self.port, self.collection,
        )

    def find(self, *args, **kwargs):
        transaction = jennifer_agent().current_transaction()
        parameter = None
        document = safe(args, 0)
        try:
            parameter = json.dumps(document)
        except:
            pass

        result = None
        if transaction is not None and parameter is not None:
            transaction.profiler.db_execute(
                self.host, self.port, self.collection + '.find(' + \
                    parameter + ')')
        try:
            result = self._origin.find(*args, **kwargs)
        except Exception as e:
            if transaction is not None and parameter is not None:
                transaction.profiler.sql_error(e)
                transaction.profiler.end()
            raise e

        if transaction is not None and parameter is not None:
            transaction.profiler.end()

        return result

    def find_one_and_replace(self, *args, **kwargs):
        transaction = jennifer_agent().current_transaction()
        parameter = None
        filter_ = safe(args, 0) or kwargs.get('filter')
        replacement = safe(args, 1) or kwargs.get('replacement')

        try:
            parameter = [json.dumps(filter_), json.dumps(replacement)]
        except:
            pass

        result = None
        if transaction is not None and parameter is not None:
            transaction.profiler.db_execute(
                self.host, self.port, self.collection + '.findOneAndReplace(' + \
                    parameter[0] + ', ' + parameter[1] + ')')
        try:
            result = self._origin.find(*args, **kwargs)
        except Exception as e:
            if transaction is not None and parameter is not None:
                transaction.profiler.sql_error(e)
                transaction.profiler.end()
            raise e

        if transaction is not None and parameter is not None:
            transaction.profiler.end()

        return result

    def insert_many(self, *args, **kwargs):
        transaction = jennifer_agent().current_transaction()
        parameter = None
        document = safe(args, 0) or kwargs.get('document')
        try:
            parameter = json.dumps(document)
        except:
            pass

        result = None
        if transaction is not None and parameter is not None:
            transaction.profiler.db_execute(
                self.host, self.port, self.collection + ".insert(" + \
                    parameter + ")")
        try:
            result = self._origin.insert_many(*args, **kwargs)
        except Exception as e:
            if transaction is not None and parameter is not None:
                transaction.profiler.sql_error(e)
                transaction.profiler.end()
            raise e

        if transaction is not None and parameter is not None:
            transaction.profiler.end()

        return result

    def update_many(self, *args, **kwargs):
        transaction = jennifer_agent().current_transaction()
        parameter = None
        document = safe(args, 0) or kwargs.get('document')
        try:
            parameter = json.dumps(document)
        except:
            pass

        result = None
        if transaction is not None and parameter is not None:
            transaction.profiler.db_execute(
                self.host, self.port, self.collection + ".update(" + \
                    parameter + ")")
        try:
            result = self._origin.update_many(*args, **kwargs)
        except Exception as e:
            if transaction is not None and parameter is not None:
                transaction.profiler.sql_error(e)
                transaction.profiler.end()
            raise e

        if transaction is not None and parameter is not None:
            transaction.profiler.end()

        return result

    def update_one(self, *args, **kwargs):
        transaction = jennifer_agent().current_transaction()
        parameter = None
        document = safe(args, 0) or kwargs.get('document')
        try:
            parameter = json.dumps(document)
        except:
            pass

        result = None
        if transaction is not None and parameter is not None:
            transaction.profiler.db_execute(
                self.host, self.port, self.collection + ".updateone(" + \
                    parameter + ")")
        try:
            result = self._origin.update_one(*args, **kwargs)
        except Exception as e:
            if transaction is not None and parameter is not None:
                transaction.profiler.sql_error(e)
                transaction.profiler.end()
            raise e

        if transaction is not None and parameter is not None:
            transaction.profiler.end()

        return result

    def insert_one(self, *args, **kwargs):
        transaction = jennifer_agent().current_transaction()
        parameter = None
        document = safe(args, 0) or kwargs.get('doc')
        try:
            parameter = json.dumps(document)
        except:
            pass

        result = None
        if transaction is not None and parameter is not None:
            transaction.profiler.db_execute(
                self.host, self.port, self.collection + ".insertOne(" + \
                    parameter + ")")
        try:
            result = self._origin.insert_one(*args, **kwargs)
        except Exception as e:
            if transaction is not None and parameter is not None:
                transaction.profiler.sql_error(e)
                transaction.profiler.end()
            raise e

        if transaction is not None and parameter is not None:
            transaction.profiler.end()

        return result


class DatabaseProxy(Proxy):
    def __init__(self, obj, host, port, name):
        Proxy.__init__(self, obj)
        self.set('db', name)
        origin_getitem = self._origin.__getitem__
        def getitem(name):
            collection = origin_getitem(name)
            return CollectionProxy(collection, host, port, self.db + '.' + name)

        self._origin.__getitem__ = getitem


def hook(pymongo):
    MongoClientOrigin = pymongo.MongoClient
    class MongoClientWrap(MongoClientOrigin):
        def __getitem__(self, name):
            database = MongoClientOrigin.__getitem__(self, name)
            host, port = self.address
            proxy = DatabaseProxy(database, host, port, name)
            return proxy
    pymongo.MongoClient = MongoClientWrap



from .. import remote_call
from .profile_data import ProfileDataType


class ExternalCall(ProfileDataType):
    TYPE_NONE = remote_call.TYPE_NONE
    TYPE_HTTP = remote_call.TYPE_HTTP
    TYPE_HTTPS = remote_call.TYPE_HTTPS

    def __init__(
            self,
            protocol='http',
            host='',
            port=80,
            text_hash='',
            desc_hash=0,
            error_hash=0,
    ):
        ProfileDataType.__init__(self)
        self.text_hash = text_hash
        protocol = protocol.lower()
        self.protocol = ExternalCall.TYPE_NONE
        if protocol == 'http':
            self.protocol = ExternalCall.TYPE_HTTP
        elif protocol == 'https':
            self.protocol = ExternalCall.TYPE_HTTPS
        self.host = host
        self.port = port
        self.desc_hash = desc_hash
        self.error_hash = error_hash

    def get_type(self):
        return ProfileDataType.TYPE_TXCALL



#!/usr/bin/env python
import sys
from .generate_config import generate_config
from .run import run


def print_help():
    print("""
Usage: jennfier <command> [options]

JENNIFER 5 python agent

Commands:
    generate-config
    version
    run
""")


def main():
    # read command
    if len(sys.argv) < 2:
        print_help()
        exit()

    command = sys.argv[1]
    try:
        if command == 'generate-config':
            generate_config(sys.argv[1:])
            exit(-1)
        elif command == 'version':
            import jennifer
            print('JENNIFER Python(%s)' % jennifer.__version__)
        elif command == 'run':
            run(sys.argv[1:])
        else:
            print_help()
    except (EnvironmentError, FileNotFoundError)as e:
        print("Error: {0}\n\ntype 'jennifer {1} help' for help".format(
            str(e), command
        ))


if __name__ == '__main__':
    main()
    exit(-1)



from jennifer.wrap.wsgi import wrap_wsgi_app
from jennifer.agent import jennifer_agent
from jennifer.api.format import format_function

__hooking_module__ = 'django'


def hook(django):
    from django.core.handlers.wsgi import WSGIHandler
    def wrapper(origin):
        def handler(self, environ, start_response):
            origin_path_info = environ.get('PATH_INFO')
            request = self.request_class(environ)
            resolver = None
            if hasattr(django, 'urls') and hasattr(django.urls, 'get_resolver'):
                get_resolver = django.urls.get_resolver
                if hasattr(request, 'urlconf'):
                    urlconf = request.urlconf
                    resolver = get_resolver(urlconf)
                else:
                    resolver = get_resolver()
            elif hasattr(django.core, 'urlresolvers'):
                urlresolvers = django.core.urlresolvers
                settings = django.conf.settings
                urlconf = settings.ROOT_URLCONF
                urlresolvers.set_urlconf(urlconf)
                resolver = urlresolvers.RegexURLResolver(r'^/', urlconf)
                if hasattr(request, 'urlconf'):
                    urlconf = request.urlconf
                    urlresolvers.set_urlconf(urlconf)
                    resolver = urlresolvers.RegexURLResolver(r'^/', urlconf)

            if resolver is not None:
                try:
                    resolver_match = resolver.resolve(request.path_info)
                    name = format_function(resolver_match.func)
                    profiler = jennifer_agent().current_transaction().profiler
                    profiler.set_root_name(name)
                except:
                    pass
            if origin_path_info is not None:
                environ['PATH_INFO'] = origin_path_info
            return origin(self, environ, start_response)
        return handler
    WSGIHandler.__call__ = wrapper(WSGIHandler.__call__)
    WSGIHandler.__call__ = wrap_wsgi_app(
        WSGIHandler.__call__,
        'django.core.handlers.wsgi.WSGIHandler.__call__')



TYPE_NONE = 0
TYPE_CUSTOM = 11
TYPE_HTTP = 12
TYPE_HTTPS = 18
TYPE_UNKNOWN_SQL_DATABASE = 21
TYPE_ORACLE = 22
TYPE_DB2 = 23
TYPE_MYSQL = 24
TYPE_POSTGRESQL = 25
TYPE_CUBRID = 26
TYPE_MSSQL = 27
TYPE_SOAP = 32

class OutRemoteCall:
    def __init__(
            self,
            call_type=TYPE_NONE,
            host='',
            port=0,
            request_hash=0,
            recv_sid=0,
            recv_oid=0,
            desc_hash=0,
    ):
        self.call_type = call_type
        self.host = host
        self.port = port
        self.request_hash = request_hash
        self.recv_sid = recv_sid
        self.recv_oid = recv_oid
        self.desc_hash = desc_hash



import inspect


def format_function(func):
    # if im_class exist it is probably unbound method
    name = ''
    if hasattr(func, 'im_class'):
        im_class = func.im_class
        name = '.'.join([
            im_class.__module__,
            im_class.__name__,
            func.__name__,
        ])
    elif hasattr(func, '__qualname__'):
        if func.__name__ == func.__qualname__:  # class name is not defined
            name = '.'.join([func.__module__, '', func.__name__])
        else:
            name = '.'.join([func.__module__, func.__qualname__])
    else:
        name = '.'.join([func.__module__, '', func.__name__])

    args = ''
    if hasattr(inspect, 'getfullargspec'):
        args = ','.join(inspect.getfullargspec(func).args)
    elif hasattr(inspect, 'getargspec'):
        args = ','.join(inspect.getargspec(func).args)

    name = '%s(%s)' % (name, args)

    return name



import traceback
import datetime
from . import jennifer_agent
from jennifer.protocol.types.profile import ProfileData, File, Socket, \
    Message, Method, Root, ExternalCall, Error, sql
from jennifer.recorder.db import DBConnectionRecorder


RUNNING_MODE_NONE = 0
RUNNING_SQL = 1
RUNNING_TXCALL = 2

STATUS_RUN = 20
STATUS_REJECTING = 21
STATUS_REJECTED = 22
STATUS_DB_CONNECTING = 50
STATUS_DB_CONNECTED = 51
STATUS_DB_STMT_OPEN = 52
STATUS_SQL_EXECUTING = 54
STATUS_SQL_EXECUTED = 55
STATUS_DB_COLOSED = 60
STATUS_TXCALL_EXECUTING = 70
STATUS_TXCALL_EXECUTED = 70
STATUS_TXCALL_END = 72

# TODO: Partial!
class TransactionProfiler(ProfileData):
    def __init__(self, transaction, service_hash):
        ProfileData.__init__(self, transaction.txid, service_hash, [])
        self.agent = jennifer_agent()
        self.root = Root(
            name_hash=self.agent.hash_text('wsgi_handler', 'method'))
        self.root.parent = None
        self.children.append(self.root)
        self.context = self.root
        self.root_start_time = self.agent.current_time()
        self.root_start_cpu = self.agent.current_cputime()
        self.last_index = 0
        self.running_mode = RUNNING_MODE_NONE
        self.running_hash = 0
        self.status = 0
        self.running_start_time = 0
        self.transaction = transaction
        self.db_recorder = DBConnectionRecorder()

    def end(self, **kwargs):
        self.record_elapsed(self.context)
        # input message case by case
        if isinstance(self.context, ExternalCall) and \
             kwargs.get('message') is not None:
                self.running_mode = RUNNING_MODE_NONE
                self.running_hash = 0
                self.status = 0
                self.running_start_time = 0
                self.message(kwargs.get('message'))
        elif isinstance(self.context, sql.Query):
            self.running_mode = RUNNING_MODE_NONE
            self.running_hash = 0
            self.status = 0
            self.running_start_time = 0
            self.transaction.sql_count += 1
            self.transaction.sql_time += self.context.elapsed_time
        elif isinstance(self.context, sql.Fetch):
            self.transaction.fetch_count += 1
            self.transaction.fetch_time += self.context.elapsed_time
        if self.context.parent is not None:
            self.switch_context(self.context.parent)

    def set_root_name(self, name):
        self.root.name_hash = self.agent.hash_text(name, 'method')

    def method(self, name, error=None):  # TODO: test it!
        method = Method(
            self.agent.hash_text(name, 'method'),
            self.agent.hash_text(error, 'method'))
        self.record_context(method)

    def external_call(self, protocol, url, host, port=80, caller=''):
        call_hash = self.agent.hash_text(
            '%s (url=%s)' % (caller, url),
            'txcall'
        )
        tx = ExternalCall(
            protocol=protocol,
            host=host,
            port=port or 80,
            text_hash=call_hash
        )
        self.running_mode = RUNNING_TXCALL
        self.status = STATUS_TXCALL_EXECUTING
        self.running_hash = call_hash
        self.running_start_time = self.agent.current_time()
        self.record_context(tx)

    def db_open(self, host, port, db):
        msg = sql.Message(
            0,
            'mysql:host={0};port={1};db={2}'.format(
                host, port, db
            ), sql.Message.TYPE_OPEN)
        self.record_context(msg)

    def _process_sql_params(self, param):
        t = type(param)
        if t is datetime.datetime:
            param = param.strftime('%Y-%m-%d %H:%M:%S')
        elif t is datetime.date:
            param = param.strftime('%Y-%m-%d')
        return param

    def db_execute(
        self, host='', port=3306, query='', params=[], style='format'):
        if type(params) is list:
            params = [self._process_sql_params(x) for x in params]
        if type(params) is dict:
            params = {
                k: self._process_sql_params(v) for (k, v) in params.items()
            }

        query_hash = self.agent.hash_text(query, 'sql')
        q = sql.Query(
            host=host,
            port=port,
            query=query,
            params=params,
            format=style,
        )
        self.running_mode = RUNNING_SQL
        self.status = STATUS_SQL_EXECUTING
        self.running_hash = query_hash
        self.running_start_time = self.agent.current_time()
        self.record_context(q)

    def db_fetch(self, size=0):
        f = sql.Fetch(size)
        self.record_context(f)

    def message(self, text):
        msg = Message(text)
        self.record(msg)

    def file_opened(self, name, mode):
        file = File(name, mode)
        self.record(file)

    def socket_opened(self, host, port, local):
        socket = Socket(host, port, local)
        self.record(socket)

    def service_error(self, exc):
        self.exception(exc, Error.SERVICE_ERROR)

    def not_found(self, exc):
        self.exception(exc, Error.HTTP_404_ERROR)

    def db_connection_error(self, exc):
        self.exception(exc, Error.DB_CONNECTION_FAIL)

    def sql_error(self, exc):
        self.exception(exc, Error.SQL_EXCEPTION)

    def exception(self, exc, error_type=None):
        message = traceback.format_exception_only(
            exc.__class__,
            exc)
        if error_type is None:
            t = type(exc)

            if hasattr(__builtins__, 'RecursionError'):  # only support python3
                if t == RecursionError:
                    error_type = RECURSIVE_CALL

            if t == MemoryError:
                error_type = Error.OUT_OF_MEMORY
            elif t == SyntaxError or t == IndentationError:
                error_type = Error.PARSE_ERROR
            elif t == SystemError or t == OSError:
                error_type = Error.NATIVE_CRITICAL_ERROR
            else:
                error_type = Error.SERVICE_EXCEPTION
        if message is not None:
            self.error(error_type, ''.join(message))

    def error(self, error_type, message):
        error_hash = self.agent.hash_text(message, 'event_detail_msg')
        error = Error(error_type, error_hash)
        self.transaction.error_code = error_type
        self.record(error)

    def record_context(self, context):
        self.children.append(context)
        self.record_index(context)
        context.parent = self.context
        self.switch_context(context)
        self.record_start(context)

    def record(self, profile):
        self.children.append(profile)
        self.record_index(profile)
        self.record_start(profile)

    def switch_context(self, profile):
        self.context = profile

    def record_start(self, profile):
        profile.start_time = self.gap_time()
        profile.start_cpu = self.gap_cputime()

    def record_index(self, profile):
        self.last_index += 1
        profile.index = self.last_index
        profile.parent_index = self.context.index

    def record_elapsed(self, profile):
        profile.elapsed_time = self.gap_time() - profile.start_time
        profile.elapsed_cpu = self.gap_time() - profile.start_cpu

    def gap_time(self):
        return self.agent.current_time() - self.root_start_time

    def gap_cputime(self):
        return self.agent.current_cputime() - self.root_start_cpu



import os

DEFAULT_CONFIG = """[JENNIFER]

server_address = 127.0.0.1
server_port = 5000
domain_id = 1000
inst_id = -1
log_path = /tmp/jennifer-python-agent.log
"""


def print_help():
    print("""
Usage: jennifer generate-config [output_file]

output_file: default output file name is jennifer.ini
""")


def generate_config(args):
    if len(args) < 2:
        filename = "jennifer.ini"
    else:
        filename = args[1]
        if filename in ['--help', '-h']:
            print_help()
            return
    if os.path.exists(filename):
        print("{0} is already exists.".format(filename))
        return
    f = open(os.path.join(os.getcwd(), filename), 'w')
    f.write(DEFAULT_CONFIG)
    f.flush()
    f.close()



"""Aries agent for python wsgi application server
Jennifer version 5 agent for python
"""

import os
from . import config
from . import wrap


__version__ = '5.3.0.12'
__author__ = 'Luavis'


def wsgi_app(app):
    if os.environ.get('JENNIFER_MASTER_ADDRESS') is not None:
        return wrap.wsgi.wrap_wsgi_app(app)
    return app



class DBConnectionRecorder(object):
    def __init__(self):
        self.connections = {}

    def add_connection(self, conn):
        if conn not in self.connections.keys():
            self.connections[conn] = 0

    def remove_connection(self, conn):
        del self.connections[conn]

    def active(self, conn):
        self.connections[conn] = 1

    def deactive(self, conn):
        self.connections[conn] = 0

    def record(self):
        active = 0
        values = self.connections.values()
        for v in values:
            active += v
        return (
            len(self.connections),  # total
            active,  # active
        )



from ..profile_data import ProfileDataType


class Fetch(ProfileDataType):
    def __init__(
            self,
            count=0,
    ):
        ProfileDataType.__init__(self)
        self.count = count

    def get_type(self):
        return ProfileDataType.TYPE_SQL_FETCH



import os
from jennifer.agent import jennifer_agent
from . import app_flask, \
    app_django, \
    db_pymysql, \
    db_sqlite3, \
    db_mysqlclient, \
    db_pymongo, \
    db_redis, \
    external_urllib, \
    external_requests

HOOK_SUPPORT_LIST = [
    app_flask,
    app_django,
    db_mysqlclient,
    db_pymysql,
    db_sqlite3,
    db_pymongo,
    db_redis,
    external_urllib,
    external_requests,
]


def is_module_exist(module):
    try:
        return __import__(module)
    except ImportError:
        return False


def wrap_open(origin):
    agent = jennifer_agent()

    def handler(file, mode='r', *args, **kwargs):
        transaction = agent.current_transaction()
        if transaction is not None:
            transaction.profiler.file_opened(
                name=os.path.abspath(os.path.join(os.getcwd(), file)),
                mode=mode
            )
        return origin(file, mode, *args, **kwargs)
    return handler


def wrap_connect(connect):
    import socket
    agent = jennifer_agent()

    def handler(self, address):
        transaction = agent.current_transaction()
        ret = connect(self, address)
        if self.family != socket.AF_INET:
            return ret
        if transaction is not None:
            raddr = self.getpeername()
            laddr = self.getsockname()
            transaction.profiler.socket_opened(
                host=raddr[0],
                port=raddr[1],
                local=laddr[1],
            )
        return ret
    return handler


def hook_builtins():
    import socket
    __builtins__['open'] = wrap_open(__builtins__['open'])
    socket.socket.connect = wrap_connect(socket.socket.connect)


def hooking():
    for m in HOOK_SUPPORT_LIST:
        module = is_module_exist(m.__hooking_module__)
        if module is not False:
            m.hook(module)
    hook_builtins()
